/**
 * @file src/DateTime.cpp
 * @author The RECORD Development Team (INRA)
 */

/*
 * Copyright (C) 2010-2012 INRA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iomanip>      // std::setprecision

#include <vle/extension/DifferenceEquation.hpp>

#include <vle/utils/DateTime.hpp>
#include <iostream>

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;
namespace vu = vle::utils;

namespace DateTime {

class DateTimeSample : public ve::DifferenceEquation::Multiple
{
public:
    DateTimeSample(
       const vd::DynamicsInit& atom,
       const vd::InitEventList& evts)
        : ve::DifferenceEquation::Multiple(atom, evts)
    {
        x = createVar("x");
    }

    virtual ~DateTimeSample()
    {}

    virtual void compute(const vd::Time& time) {

        std::cout << "------------------------------------------------ " << std::endl;

        std::cout << "time: " << std::setprecision(20) << time << std::endl;

        std::cout << "currentDate(): " << vu::DateTime::currentDate() << std::endl;

        std::cout << "year(time): " << vu::DateTime::year(time) << std::endl;

        std::cout << "month(time): " << vu::DateTime::month(time) << std::endl;

        std::cout << "dayOfMonth(time): " << vu::DateTime::dayOfMonth(time) << std::endl;

        std::cout << "dayOfWeek(time): " << vu::DateTime::dayOfWeek(time) << std::endl;

        std::cout << "dayOfYear(time): " << vu::DateTime::dayOfYear(time) << std::endl;

        std::cout << "weekOfYear(time): " << vu::DateTime::weekOfYear(time) << std::endl;

        std::cout << "isLeapYear(time): " << vu::DateTime::isLeapYear(time) << std::endl;

        std::cout << "aYear(time): " << vu::DateTime::aYear(time) << std::endl;

        std::cout << "aMonth(time): " << vu::DateTime::aMonth(time) << std::endl;

        std::cout << "aWeek(): " << vu::DateTime::aWeek() << std::endl;

        std::cout << "aDay(): " << vu::DateTime::aDay() << std::endl;

        std::cout << "years(time, 1): " << vu::DateTime::years(time, 1) << std::endl;

        std::cout << "months(time, 1): " << vu::DateTime::months(time, 1) << std::endl;

        std::cout << "weeks(1): " << vu::DateTime::weeks(1) << std::endl;

        std::cout << "days(1): " << vu::DateTime::days(1) << std::endl;

        std::cout << "days(1.5): " << vu::DateTime::days(1.5) << std::endl;

        std::cout << "convertUnit('day'): " << enum2str(vu::DateTime::convertUnit("day")) << std::endl;

        std::cout << "convertUnit('week'): " << enum2str(vu::DateTime::convertUnit("week")) << std::endl;

        std::cout << "convertUnit('month'): " << enum2str(vu::DateTime::convertUnit("month")) << std::endl;

        std::cout << "convertUnit('year'): " << enum2str(vu::DateTime::convertUnit("year")) << std::endl;

        std::cout << "duration(time, 10, convertUnit('day')): " << vu::DateTime::duration(time, 10, vu::DateTime::convertUnit("day")) << std::endl;

        std::cout << "duration(time, 10, convertUnit('week')): " << vu::DateTime::duration(time, 10, vu::DateTime::convertUnit("week")) << std::endl;

        std::cout << "duration(time, 10, convertUnit('month')): " << vu::DateTime::duration(time, 10, vu::DateTime::convertUnit("month")) << std::endl;

        std::cout << "duration(time, 10, convertUnit('year')): " << vu::DateTime::duration(time, 10, vu::DateTime::convertUnit("year")) << std::endl;

        std::cout << "toJulianDayNumber(time): " << vu::DateTime::toJulianDayNumber(time) << std::endl;

        std::cout << "toJulianDayNumber(toJulianDayNumber(time)): " << vu::DateTime::toJulianDayNumber(vu::DateTime::toJulianDayNumber(time)) << std::endl;

        std::cout << "toJulianDay(time): " << vu::DateTime::toJulianDay(time) << std::endl;

        std::cout << "toJulianDay(toJulianDay(time)): " << vu::DateTime::toJulianDay(vu::DateTime::toJulianDay(time)) << std::endl;

        std::cout << "isValidYear(time): " << vu::DateTime::isValidYear(time) << std::endl;

        long year, month, day, hours, minutes, seconds;
        std::cout << "toTime(time+10, year, month, day, hours, minutes, seconds): " << vu::DateTime::toTime(time+10, year, month, day, hours, minutes, seconds) << std::endl;
        std::cout << "seconds: " << seconds << std::endl;

        x = time;
    }

    virtual void initValue(const vd::Time& /*time*/) { }

private:

    enum Unit { None, Day, Week, Month, Year };

    char* enum2str(vu::DateTimeUnitOptions u) {
        switch (u) {
            case vu::DATE_TIME_UNIT_NONE:
                return (char*)&"None";
            case vu::DATE_TIME_UNIT_DAY:
                return (char*)&"Day";
            case vu::DATE_TIME_UNIT_WEEK:
                return (char*)&"Week";
            case vu::DATE_TIME_UNIT_MONTH:
                return (char*)&"Month";
            case vu::DATE_TIME_UNIT_YEAR:
                return (char*)&"Year";
            default:
                return (char*)&"INVALID ENUM";
        }
    }

    Var x;
};

} // namespace DateTime

DECLARE_DYNAMICS(DateTime::DateTimeSample)

